/*
  Copyright 2005, 2006 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <limits.h>
#include <gmp.h>

#ifndef NOT_USING_MPFR
#include <mpfr.h>
#endif

#ifndef USE_DOUBLE
#ifndef USE_DPE
#ifndef USE_MPFR
#ifndef USE_LONGDOUBLE
#define USE_DPE
#endif
#endif
#endif
#endif

#include "intclass.h"
#include "fpclass.h"
#include "basic.h"

FP_NR*
init_vecf (int n)
{
  FP_NR *a;
  int i;

  a = (FP_NR*) malloc (n * sizeof (FP_NR));

  for (i=0; i<n; i++)
    INIT (a[i]);
  return a;
}

void
clear_vecf (FP_NR *a, int n)
{
  int i;

  for (i=0; i<n; i++)
    CLEAR (a[i]);

  free (a);
}

void
Print_vecf (FP_NR *B, int n) 
{
  int i; 

  printf("[");
  for (i=1;i<=n;i++) {
    OUT(B[i]); printf(" ");
  }
  printf("]\n"); 
}

void
init_matrixf (mat_RR *B, int d, int n)
{
  int i;

  B->NumCols=d;
  B->NumRows=n; 
  B->coeff = (FP_NR**) malloc ((d + 1) * sizeof (FP_NR*));
  for (i=0; i<=d; i++)
    B->coeff[i] = init_vecf (n + 1);
}


void
clear_matrixf (mat_RR *B, int d, int n)
{
  int i;
  for (i=0; i<=d; i++)
    clear_vecf (B->coeff[i], n + 1);
  free (B->coeff);
}

void
Print_matf (FP_NR **B, int d, int n) 
{
  int i, j; 

  printf("[");
  for (i=1;i<=d;i++) {
    printf("[");
    for (j=1;j<=n;j++) { 
      OUT (B[i][j]); 
      if (j < n) printf(" "); 
    }
    printf("]\n"); 
  }  
  printf("]\n"); 
}
