/*
  Copyright 2005 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/


#include <math.h>
#include <float.h>
#include <gmp.h>

#ifndef NOT_USING_MPFR
#define USING_MPFR
#endif
#ifdef USING_MPFR
#include <mpfr.h>
#endif

#if (LONG_MAX==2147483647L)
#define EXPO_MAX 30
#define EXPO_MAX2 30
#else
#define EXPO_MAX 53
#define EXPO_MAX2 62
#endif

#include "intclass.h"
#include "fpclass.h"


/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*% USE_MPFR *%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */

#ifdef USE_MPFR
#ifdef ZUSE_INT
long int 
mpfr_get_si_exp (int *rop, mpfr_t op)
{
  long int expo;
  double f;

  f = mpfr_get_d_2exp (&expo, op, GMP_RNDN);
  if (expo<=30){
    rop[0] = (int) ldexp(f, expo);
    expo=0;
  }
  else{
    rop[0] = (int) ldexp(f, 30);
    expo-=30;
  }
  return expo;
}

#endif

/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*% USE_DOUBLE *%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */

#else
#ifdef USE_DOUBLE

long int
double_get_si_exp (long int *x, double *y)
{
  long int expo = ilogb (y[0]);
  
  if (expo < EXPO_MAX){
    expo = 0;
    x[0] = (long int) y[0];
  }
  else{
    expo -= EXPO_MAX;
    x[0] = (long int) (ldexp(y[0],-expo));
  }
  return expo;
}

/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*% USE_DPE %*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
#else
#ifdef USE_DPE
#ifdef ZUSE_MPFR
//TODO : does not use all that could be ifdef FORCE_LONGDOUBLE

void 
dpe_set_mpfr (dpe_t a, mpfr_t b)
{
  mp_exp_t expo;
  double tmp;
  
  tmp = mpfr_get_d_2exp (&expo, b, GMP_RNDN);
  dpe_set_d (a, tmp);
  dpe_mul_2exp (a, a, expo);
}

#endif

/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*% USE_LONGDOUBLE *%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */


#else
#ifdef USE_LONGDOUBLE

long int
long_double_get_si_exp (long int *x, long double *y)
{
  long int expo = ilogbl (y[0]);
  if (expo < EXPO_MAX2){
    expo = 0;
    x[0]= (long int) y[0];
  }
  else{
    expo -=EXPO_MAX2;
    x[0] = (long int) (ldexpl(y[0],-expo));
  }
  return expo;
}




#ifdef ZUSE_MPZ
long double 
mpz_get_ld (mpz_t a)
{
  long double res;
  double tmp1, tmp2;
  mpz_t ztmp;
  
  mpz_init (ztmp);
  tmp1 = mpz_get_d (a);
  mpz_set_d (ztmp, tmp1);
  mpz_sub (ztmp, a, ztmp);
  tmp2 = mpz_get_d (ztmp);
  res = ((long double) tmp1) + ((long double) tmp2);

  mpz_clear (ztmp);
  return res;
}

#endif
#endif
#endif
#endif
#endif
