/*
  Copyright 2005, 2006 Damien Stehlé.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/


#include <stdlib.h>
#include <math.h>
#include <gmp.h>

#ifdef ZUSE_MPFR
#include <mpfr.h>
#endif

#include "intclass.h"

#ifdef ZUSE_INT

int
int_sizeinbase2 (int* x)
{
  int resul=1, y=abs(x[0]);

  if (y==0) resul=1;
  else{
    while (y>1){
      resul++;
      y /=2;
    }
  }    
  return resul;
}

double 
int_get_d_2exp (signed long int* exp, int* op)
{
  if (op[0]==0) exp[0] = 0;
  else exp[0] = int_sizeinbase2 (op);
  return ldexp( (double) op[0], -exp[0]);
}

int 
int_sgn (int op)
{
  int result=0;
  if (op>0) result=1;
  if (op<0) result=-1;
  return result;
}


void 
int_urandomb(int* op, unsigned long int n)
{
  op[0] = (int) (ldexp((double) rand(), -31+n));
}

void 
int_urandomm(int* op, unsigned long int n)
{
  op[0] = rand() % n;
}

/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */

#else
#ifdef ZUSE_MPFR

void 
mpfr_tdiv_r_2exp (mpfr_t a, mpfr_t b, int c)
{
  mpfr_t tmp;
  mpfr_init (tmp);

  mpfr_div_2exp (tmp, b, c, GMP_RNDN);
  mpfr_frac (tmp, tmp, GMP_RNDN);
  mpfr_mul_2exp (a, tmp, c, GMP_RNDN);

  mpfr_clear (tmp);

}


void 
mpfr_submul_ui (mpfr_t a, mpfr_t b, unsigned long int c, mp_rnd_t rn)
{
  mp_prec_t prec;
  mpfr_t rtmp;

  prec = mpfr_get_prec (a);

  mpfr_init2 (rtmp, prec);
  mpfr_mul_ui (rtmp, b, c, rn);
  mpfr_sub (a, a, rtmp, rn);
  mpfr_clear (rtmp);
}


void 
mpfr_addmul_ui (mpfr_t a, mpfr_t b, unsigned long int c, mp_rnd_t rn)
{
  mp_prec_t prec;
  mpfr_t rtmp;

  prec = mpfr_get_prec (a);

  mpfr_init2 (rtmp, prec);
  mpfr_mul_ui (rtmp, b, c, rn);
  mpfr_add (a, a, rtmp, rn);
  mpfr_clear (rtmp);
}



void 
mpfr_urandomb2 (mpfr_t rop, ZRANDSTATE state, unsigned long int bits)
{
  mpz_t ztmp;
  
  mpz_init (ztmp);
  mpz_urandomb (ztmp, state, bits);
  mpfr_set_z (rop, ztmp, GMP_RNDN);
  
  mpz_clear (ztmp);
}


void 
mpfr_urandomm2 (mpfr_t rop, ZRANDSTATE state, mpfr_t op)
{
  mpz_t ztmp;
  
  mpz_init (ztmp);
  mpfr_get_z (ztmp, op, GMP_RNDN);
  mpz_urandomm (ztmp, state, ztmp);
  mpfr_set_z (rop, ztmp, GMP_RNDN);
  
  mpz_clear (ztmp);
}


#endif
#endif
