/*
  Copyright 2005, 2006 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

/* Choose your integer class */

// #define ZUSE_MPZ
// #define ZUSE_INT
// #define ZUSE_MPFR
// #define ZUSE_DPE
 

#ifdef ZUSE_MPZ

#define Z_NR mpz_t
#define ZINIT(a) mpz_init(a)
#define ZCLEAR(a) mpz_clear(a)
#define ZOUT(a) mpz_out_str (stdout, 10, a)
#define ZOUT2(a) mpz_out_str (stderr, 10, a)
#define ZIN(a) mpz_inp_str(a, stdin, 0);

#define ZSET(a,b) mpz_set(a, b)
#define ZSET_UI(a,b) mpz_set_ui(a, b)
#define ZSIZEINBASE2(a) mpz_sizeinbase(a, 2)
#define ZGET_D_2EXP(a,b,c) a = mpz_get_d_2exp (b, c) 

#define ZTDIV_R_2EXP(a,b,c) mpz_tdiv_r_2exp (a, b, c)

#ifdef USING_MPFR
#define ZMPFR_GET_Z(a,b) mpfr_get_z (a, b, GMP_RNDN)
#endif

#define ZADD(a,b,c) mpz_add (a, b, c)
#define ZSUB(a,b,c) mpz_sub (a, b, c)
#define ZMUL(a,b,c) mpz_mul (a, b, c)
#define ZSGN(a) mpz_sgn(a)
#define ZABS(a,b) mpz_abs(a, b)
#define ZCMP(a,b) mpz_cmp(a, b)
#define ZNEG(a,b) mpz_neg(a, b)

#define ZADD_UI(a,b,c) mpz_add_ui(a, b, c)
#define ZSUBMUL_UI(a,b,c) mpz_submul_ui(a, b, c)
#define ZADDMUL_UI(a,b,c) mpz_addmul_ui(a, b, c)
#define ZMUL_SI(a,b,c) mpz_mul_si(a, b, c)
#define ZMUL_UI(a,b,c) mpz_mul_ui(a, b, c)
#define ZMUL_2EXP(a,b,c) mpz_mul_2exp (a, b, c)
#define ZDIV_2EXP(a,b,c) mpz_div_2exp(a, b, c)

#define MP_EXP_T mp_exp_t

#define ZRANDSTATE gmp_randstate_t
#define ZRANDINIT(a) gmp_randinit_default (a)
#define ZRANDB(a,b,c) mpz_urandomb(a, b, c)
#define ZRANDM(a,b,c) mpz_urandomm(a, b, c)

#else

/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */


#ifdef ZUSE_INT

typedef int int_t[1];

#define Z_NR int_t

#define ZINIT(a) {}
#define ZCLEAR(a) {}
#define ZOUT(a) fprintf (stdout, "%d", a[0])
#define ZOUT2(a) fprintf (stderr, "%d", a[0])
#define ZIN(a) fscanf (stdin, "%d", a);

#define ZSET(a,b) a[0]=b[0]
#define ZSET_UI(a,b) a[0] = b

int int_sizeinbase2 (int*);
double int_get_d_2exp (signed long int*, int*);

#define ZSIZEINBASE2(a) int_sizeinbase2(a)
#define ZGET_D_2EXP(a,b,c) a = int_get_d_2exp (b, c) 

#define ZTDIV_R_2EXP(a,b,c) a = b % (1<<c)

#ifdef USING_MPFR
#define ZMPFR_GET_Z(a,b) a[0] = (int) mpfr_get_si (b, GMP_RNDN)
#endif

int int_sgn (int);

#define ZADD(a,b,c) a[0]= b[0]+c[0]
#define ZSUB(a,b,c) a[0]= b[0]-c[0]
#define ZMUL(a,b,c) a[0]= b[0]*c[0]
#define ZSGN(a) int_sgn(a[0])
#define ZABS(a,b) a[0] = abs(b[0])
#define ZCMP(a,b) int_sgn(a[0]-b[0])
#define ZNEG(a,b) a[0] = -b[0]

#define ZADD_UI(a,b,c) a[0]=b[0]+c
#define ZSUBMUL_UI(a,b,c) a[0] -= b[0]*c
#define ZADDMUL_UI(a,b,c) a[0] += b[0]*c
#define ZMUL_SI(a,b,c) a[0]=b[0]*c
#define ZMUL_UI(a,b,c) a[0]=b[0]*c
#define ZMUL_2EXP(a,b,c) a[0]=ldexp(b[0],c)
#define ZDIV_2EXP(a,b,c) a[0]=ldexp(b[0],-c)

#define MP_EXP_T long int

void int_urandomb(int*, unsigned long int);
void int_urandomm(int*, unsigned long int);

#define ZRANDSTATE int
#define ZRANDINIT(a) srand (666)
#define ZRANDB(a,b,c) int_urandomb(a, c)
#define ZRANDM(a,b,c) int_urandomm(a, c[0])

/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */
/* *%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*% */


#else
#ifdef ZUSE_MPFR

#define Z_NR mpfr_t
#define ZINIT(a) mpfr_init2(a, intprec)
#define ZCLEAR(a) mpfr_clear(a)
#define ZOUT(a) mpfr_out_str (stdout, 10, 10, a, GMP_RNDN)
#define ZOUT2(a) mpfr_out_str (stderr, 10, 5, a, GMP_RNDN)
#define ZIN(a) mpfr_inp_str(a, stdin, 10, GMP_RNDN)

#define ZSET(a,b) mpfr_set (a, b, GMP_RNDN)
#define ZSET_UI(a,b) mpfr_set_ui (a, b, GMP_RNDN)
#define ZGET_D_2EXP(a,b,c) a = mpfr_get_d_2exp (b, c, GMP_RNDN) 

#define ZMPFR_GET_Z(a,b) mpfr_rint (a, b, GMP_RNDN)

void mpfr_tdiv_r_2exp (mpfr_t, mpfr_t, int);

#define ZTDIV_R_2EXP(a,b,c) mpfr_tdiv_r_2exp (a, b, c)

#define ZADD(a,b,c) mpfr_add (a, b, c, GMP_RNDN)
#define ZSUB(a,b,c) mpfr_sub (a, b, c, GMP_RNDN)
#define ZMUL(a,b,c) mpfr_mul (a, b, c, GMP_RNDN)
#define ZSGN(a) mpfr_sgn (a)
#define ZABS(a,b) mpfr_abs (a, b, GMP_RNDN)
#define ZCMP(a,b) mpfr_cmp (a, b)
#define ZNEG(a,b) mpfr_neg (a, b, GMP_RNDN)

void mpfr_submul_ui (mpfr_t, mpfr_t, unsigned long int, mp_rnd_t);
void mpfr_addmul_ui (mpfr_t, mpfr_t, unsigned long int, mp_rnd_t);

#define ZADD_UI(a,b,c) mpfr_add_ui(a, b, c, GMP_RNDN)
#define ZSUBMUL_UI(a,b,c) mpfr_submul_ui(a, b, c, GMP_RNDN)
#define ZADDMUL_UI(a,b,c) mpfr_addmul_ui(a, b, c, GMP_RNDN)
#define ZMUL_SI(a,b,c) mpfr_mul_si(a, b, c, GMP_RNDN)
#define ZMUL_UI(a,b,c) mpfr_mul_ui(a, b, c, GMP_RNDN)
#define ZMUL_2EXP(a,b,c) mpfr_mul_2si (a, b, c, GMP_RNDN)
#define ZDIV_2EXP(a,b,c) mpfr_div_2si (a, b, c, GMP_RNDN); mpfr_rint (a, a, GMP_RNDN)

#define MP_EXP_T long int


#define ZRANDSTATE gmp_randstate_t
#define ZRANDINIT(a) gmp_randinit_default (a)

void mpfr_urandomb2 (mpfr_t, ZRANDSTATE, unsigned long int);
void mpfr_urandomm2 (mpfr_t, ZRANDSTATE, mpfr_t);

#define ZRANDB(a,b,c) mpfr_urandomb2 (a, b, c)
#define ZRANDM(a,b,c) mpfr_urandomm2 (a, b, c)

#endif
#endif
#endif
