/*
  Copyright 2005, 2006 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <limits.h>
#include <gmp.h>

#ifdef ZUSE_MPFR
#include <mpfr.h>
mp_prec_t intprec;
#endif

#include "intclass.h"
#include "mat.h"


Z_NR*
init_vec (int n)
{
  Z_NR *a;
  int i;

  a = (Z_NR*) malloc (n * sizeof (Z_NR));
  for (i=0; i<n; i++)
    ZINIT (a[i]);
  return a;
}

void
clear_vec (Z_NR *a, int n)
{
  int i;

  for (i=0; i<n; i++)
    ZCLEAR (a[i]);
  free (a);
}

void
Print_vec (Z_NR *B, int n) 
{
  int i; 

  fprintf(stderr, "[");
  for (i=1;i<=n;i++) {
    ZOUT2 (B[i]);
    fprintf(stderr," ");
  }
  fprintf(stderr, "]\n"); 
}

void
init_matrix (mat_ZZ *B, int d, int n)
{
  int i;

  B->NumCols=d;
  B->NumRows=n; 
  B->coeff = (Z_NR**) malloc ((d + 1) * sizeof (Z_NR*));
  for (i=0; i<=d; i++)
    B->coeff[i] = init_vec (n + 1);
}


void
clear_matrix (mat_ZZ *B, int d, int n)
{
  int i;
  for (i=0; i<=d; i++)
    clear_vec (B->coeff[i], n + 1);
  free (B->coeff);
}

void
Print_mat (Z_NR **B, int d, int n) 
{
  int i, j; 

  printf("[");
  for (i=1;i<=d;i++) 
    {
      printf("[");
      for (j=1;j<=n;j++) 
	{ 
	  ZOUT (B[i][j]); 
	  if (j < n) printf(" "); 
	}
      printf("]\n"); 
    }  
  printf("]\n"); 
}

/* Generate the list of the entries of the matrix.
   Magma usage, if stored in the file mat:
   load("mat");
   LLL(B);
*/

void
Print_magma (Z_NR **B, int d, int n) 
{
  int i, j; 
  fprintf (stdout, "SetVerbose (\"LLL\", 2); B:=Matrix(%d,%d,[",d, n);
  for (i=1;i<=d;i++){
    for (j=1;j<=n;j++){ 
      ZOUT (B[i][j]); 
      if ((i<d) || (j<n)) fprintf(stdout,",");
    }
  }  
  fprintf (stdout, "]);\n");
}


/* ScalarProduct of two vectors of length n */
void 
ScalarProduct (Z_NR s, Z_NR *vec1, Z_NR *vec2, int n)
{
  int i;
  Z_NR tmp;
  ZINIT (tmp);

  ZMUL (s, vec1[1], vec2[1]);
  for (i=2; i<=n; i++)
    {
      ZMUL (tmp, vec1[i], vec2[i]);
      ZADD (s, s, tmp);      
    }
  
  ZCLEAR (tmp);
} 

/* Computes the Gram matrix G of the integral basis B */

void 
Gram (mat_ZZ *G, mat_ZZ *B)
{
  int i,j;
  Z_NR tmp;
  int n = B->NumRows;
  int d = B->NumCols;

  ZINIT (tmp);

  G->NumRows = G->NumCols = d;
  for (i=1; i<=d; i++)
    {
      for(j=1; j<=i; j++)
	{
	  ScalarProduct(tmp, B->coeff[i], B->coeff[j], n);
	  ZSET(G->coeff[i][j], tmp); 
	}
    }

  ZCLEAR (tmp);
}
